html.wait, html.wait * { cursor: wait !important; }

div.ajax-tooltip {
    max-width: 600px;
    background-color: #fff;
    color: #444;
    border: 1px solid #aaa;
}

div.tooltip.show {
    opacity: 1;
}

.drag_highlight {
    background-color: #e5f3ff !important;
}

.relative_overflow {
    position: relative;
    overflow: hidden;
}

.upload-dropzone{
    background-color: #F0F0F0;
    border: 1px dashed #0f0f0f;
    display: flex;
    justify-content: center;
    align-items: center;
}

.file-upload-form {
    position: absolute;
    right: 0px;
    top: 0px;
    cursor: hand;
    font-size: 200px;
    opacity: 0;
}
